#include <stdio.h>
#include "menu.h"

typedef enum {
    INTYPE_NONE,
    INTYPE_ENUM,
    INTYPE_NUMBER,
    INTYPE_STRING
} intype_t;

typedef struct {
    struct menuitem *root;
    struct menuitem *curitem;
    intype_t intype;
    char buf[128];
} config_t;

void print_siblings(struct menuitem *root, struct menuitem *item)
{
struct menuitem *p;
unsigned i = 1;

    for(p = menu_first_sibling(root, item); !menu_last_sibling(p); p++)
        printf("%c%2d. %s\n", (p == item ? '*' : ' '), i++, p->name);
}

void print_r(struct menuitem *root, unsigned indent)
{
struct menuitem *p;
unsigned i;

    for(p = root; p && p->name; p++) {
        for(i = 0; i < indent; i++)
            printf("   ");
        puts(p->name);
        print_r(p->childs, indent + 1);
    }
}

void print_item_name(void *p)
{
config_t *config = (config_t*)p;

    printf("\nMENU HANDLER: %s\n", config->curitem->name);
}

void choose_enum(void *p)
{
config_t *config = (config_t*)p;

    if(!menu_root(config->curitem)) {
        unsigned id = config->curitem - menu_first_child(config->curitem->parent);
        *((unsigned*)config->buf) = id;
        config->intype = INTYPE_ENUM;
        config->curitem = config->curitem->parent;
                
        printf("\nENUM CHOSEN: %d\n", id);
    }
}

void input_number(void *p)
{
config_t *config = (config_t*)p;
int val;

    fgets(config->buf, sizeof(config->buf) / sizeof(config->buf[0]), stdin);
    sscanf(config->buf, "%d", &val);
    *((int*)config->buf) = val;
    config->intype = INTYPE_NUMBER;
    
    printf("\nInput number was: `%d'\n", val);
}

void input_string(void *p)
{
config_t *config = (config_t*)p;

    fgets(config->buf, sizeof(config->buf) / sizeof(config->buf[0]), stdin);
    config->intype = INTYPE_STRING;
    printf("\nInput string was: `%s'\n", config->buf);
}

int main()
{
struct menuitem menu[] = {
    { "Status", 0, 0, print_item_name },
    { "Setup", 0, (struct menuitem[]) {
        { "Type", 0, (struct menuitem[]) {
            { "Type I", 0, 0, choose_enum },
            { "Type II", 0, 0, choose_enum },
            { "Type III", 0, 0, choose_enum },
            { 0, 0, 0, 0 }
        }, 0 },
        { "Your age", 0, 0, input_number },
        { "Your name", 0, 0, input_string },
        { "Network", 0, (struct menuitem[]) {
            { "Address", 0, 0, print_item_name },
            { "Netmask", 0, 0, print_item_name },
            { "Backdoor", 0, 0, print_item_name },
            { 0, 0, 0, 0 }
        }, 0},
        { 0, 0, 0, 0 }
    }},
    { "Exit", 0, 0, print_item_name },
    { 0, 0, 0, 0 }
};
config_t config = { menu, menu, INTYPE_NONE, "" };
struct menuitem *p;
char command;

    menu_init(menu);
    /*print_r(menu, 0);*/
    
    do {
        print_siblings(config.root, config.curitem);
        
        /* Command choice */
        printf("Command (q=quit, n=next, p=prev, u=up, s=submenu/select)? ");
        fflush(stdout);
        command = getchar();
        while(command != '\n' && getchar() != '\n');
        
        switch(command) {
            case 's':
                if(menu_leaf(config.curitem)) {
                    if(config.curitem->handler)
                        config.curitem->handler(&config);
                    else
                        printf("\nHandler not defined for `%s'\n", config.curitem->name);
                }
                else
                    config.curitem = menu_first_child(config.curitem);
                break;
            case 'u':
                if(!menu_root(config.curitem))
                    config.curitem = menu_parent(config.curitem);
                else
                    printf("\nTry again, asshole.\n");
                break;
            case 'n':
                p = menu_next_sibling(menu, config.curitem);
                if(p)
                    config.curitem = p;
                else
                    printf("\nRU kidding, man?!\n");
                break;
            case 'p':
                p = menu_prev_sibling(menu, config.curitem);
                if(p)
                    config.curitem = p;
                else
                    printf("\nNo way!\n");
                break;
            case 'q':
                /* nop */
                break;
            default:
                printf("\nWhat?!\n");
                break;
        }
        printf("\n");
    } while(command != 'q');
    return 0;
}

